"""Call the pref.ai api"""
import os
import requests
import json
from typing import Optional, Dict

class PrefaiError(Exception):
    def __init__(
        self,
        code=None,
        detail=None,
        headers=None,
    ):
        self.code = code
        self.detail = detail
        self.headers = headers

    def __str__(self):
        return f"Pref.ai request failed [{self.code}]: {self.detail}"


class PrefaiTimeoutError(PrefaiError):
    pass

class PrefaiClient:
    _api_base_url = 'https://api.pref.ai'
    # _api_base_url = 'http://127.0.0.1:8000'

    def __init__(self, api_key: str = None):
        if api_key:
            self.api_key = api_key
        else:
            self.api_key = os.getenv('PREFAI_API_KEY')

    def add_record(self,
        user_action: str,
        context: Optional[str] = None,
        user_email: Optional[str] = None,
        user_id: Optional[str] = None,
        interaction_title: Optional[str] = None,
        observation: Optional[str] = None,
        create_observation: Optional[bool] = True,
    ) -> Dict:
        """
        """

        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

        try:
            response = requests.post(
                f"{self._api_base_url}/v1/records/add",
                json={
                    "user_email": user_email,
                    "user_id": user_id,
                    "user_action": user_action,
                    "context": context,
                    "interaction_title": interaction_title,
                    "observation": observation,
                    "create_observation": create_observation,
                },
                headers=headers,
                timeout=10
            )

            response.raise_for_status()
            json_response = response.json()

            return json_response
            # return {
            #     "index": json_response["index"],
            #     "expected_cost": json_response["expected_cost"],
            # }

        except requests.exceptions.Timeout:
            raise PrefaiTimeoutError("Request timed out")

        except requests.HTTPError as http_err:
            # Get details:
            detail = None
            json_body = None
            response_content = response.content.decode()
            if response_content:
                json_body = json.loads(response_content)
                detail = json_body.get('detail')
            # Construct PrefaiError
            raise PrefaiError(
                code=http_err.response.status_code,
                detail=detail,
                headers=http_err.response.headers or {},
            )


    def retrieve_records(self,
        user_action: str,
        context: Optional[str] = None,
        user_email: Optional[str] = None,
        user_id: Optional[str] = None,
        max_results: Optional[int] = 3,
        min_similarity: Optional[float] = None,
    ) -> Dict:

        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

        try:
            response = requests.post(
                f"{self._api_base_url}/v1/records/retrieve",
                json={
                    "context": context,
                    "user_action": user_action,
                    "user_email": user_email,
                    "user_id": user_id,
                    "max_results": max_results,
                    "min_similarity": min_similarity,
                },
                headers=headers,
                timeout=10
            )

            response.raise_for_status()
            json_response = response.json()

            return json_response

        except requests.exceptions.Timeout:
            raise PrefaiTimeoutError("Request timed out")

        except requests.HTTPError as http_err:
            response_content = response.content.decode()
            if response_content:
                response_dict = json.loads(response_content)
                if 'detail' in response_dict:
                    raise PrefaiError(f"Request failed: {http_err}. Reason: {response_dict['detail']}")
            raise PrefaiError(f"Request failed: {http_err}")

    def pref_prompt(self,
        user_action: str,
        context: Optional[str] = None,
        user_email: Optional[str] = None,
        user_id: Optional[str] = None,
        max_tokens: Optional[int] = 300,
        min_similarity: Optional[float] = None,
    ) -> Dict:

        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

        try:
            response = requests.post(
                f"{self._api_base_url}/v1/prompt/new",
                json={
                    "context": context,
                    "user_action": user_action,
                    "user_email": user_email,
                    "user_id": user_id,
                    "max_tokens": max_tokens,
                    "min_similarity": min_similarity,
                },
                headers=headers,
                timeout=10
            )

            response.raise_for_status()
            json_response = response.json()

            return json_response

        except requests.exceptions.Timeout:
            raise PrefaiTimeoutError("Request timed out")

        except requests.HTTPError as http_err:
            response_content = response.content.decode()
            if response_content:
                response_dict = json.loads(response_content)
                if 'detail' in response_dict:
                    raise PrefaiError(f"Request failed: {http_err}. Reason: {response_dict['detail']}")
            raise PrefaiError(f"Request failed: {http_err}")

# Initialize client
prefai_client = PrefaiClient()



from prefai import prefai_client, PrefaiError
import os

test_user_email = os.getenv("PREFAI_TEST_USER_EMAIL")

def test_add_record_bad_api_key():
    orig_api_key = prefai_client.api_key 
    prefai_client.api_key = "BAD KEY"
    success = False
    try:
        prefai_client.add_record(
            user_email = test_user_email,
            user_action = "Does this recipe have tree nuts? Can you modify if so?",
        )
        success = True
    except PrefaiError as e:
        assert e.code == 401
        assert e.detail == 'Invalid API key'
    assert not success
    prefai_client.api_key = orig_api_key

def test_add_record_none_api_key():
    orig_api_key = prefai_client.api_key 
    prefai_client.api_key = None
    success = False
    try:
        prefai_client.add_record(
            user_email = test_user_email,
            user_action = "Does this recipe have tree nuts? Can you modify if so?",
        )
        success = True
    except PrefaiError as e:
        assert e.code == 401
        assert e.detail == 'Invalid API key'
    assert not success
    prefai_client.api_key = orig_api_key

def test_add_record_missing_user():
    success = False
    try:
        prefai_client.add_record(
            # Missing user_email or user_id
            user_action = "Does this recipe have tree nuts? Can you modify if so?",
        )
        success = True
    except PrefaiError as e:
        assert e.code == 422
        assert e.detail == "Either user_email or user_id must be provided (but not both)"
    assert not success

def test_add_record_missing_source():
    success = False
    try:
        prefai_client.add_record(
            user_email = test_user_email,
            # Missing source
        )
        success = True
    except TypeError as e:
        assert e.args[0] == "add_record() missing 1 required positional argument: 'user_action'"
    assert not success


def test_add():
    res = prefai_client.add_record(
        user_email = test_user_email,
        user_action = "Does this recipe have tree nuts? Can you modify if so?",
    )
    assert res.get('success')
    assert 'record_id' in res

def test_add_with_context():
    res = prefai_client.add_record(
        user_email = test_user_email,
        context = """User: How do I ask someone's name in French?
        
        AI: You would ask: "Comment vous appelez-vous?" """,
        user_action = "Is that a casual way to ask? I don't want to sound too formal.",
    )
    assert res.get('success')
    assert 'record_id' in res

def test_add_record_with_unknown_email():
    user_email = "unknown-email@pref.ai"
    res = prefai_client.add_record(
        user_email = user_email,
        user_action = "Does this recipe have tree nuts? Can you modify if so?",
    )
    assert res.get('success')
    assert 'record_id' in res

def test_retrieve():
    res = prefai_client.retrieve_records(
        user_email = test_user_email,
        context = "User",
        user_action = "How do you say I love you in Spanish?",
        min_similarity = -1.0
    )
    assert res.get('success')
    assert res['records'][0]['user_action'].startswith('Is that a casual way to ask?')
    print(res['records'])

def test_pref_prompt():
    res = prefai_client.pref_prompt(
        user_email = test_user_email,
        context = "User: Hola.\nAI: Hola!",
        user_action = "How do you say I love you in Spanish?",
        min_similarity = -1.0
    )
    assert res.get('success')
    return res
    # assert res['records'][0]['user_action'].startswith('Is that a casual way to ask?')
    # print(res['records'])

if __name__ == "__main__":
    res = test_pref_prompt()



from setuptools import setup, find_packages

setup(
    name="prefai",
    version="0.0.5",
    packages=find_packages(),
    author="Bobby Jaros",
    author_email="bobby@pref.ai",
    description="A brief description of your package",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="http://github.com/prefai/pythonclient",
    license='LICENSE.txt',
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    install_requires=[
        # Add your project dependencies here
    ],
    python_requires='>=3.6',
    include_package_data=True
)
